#include <avr32/io.h>
#include "smc_at32ap7000.h"

void  SMC_PreConfig (void *paddr, BSP_SMC_HDR *settings, unsigned int hsb_clock, unsigned long *ret)
{
	volatile	unsigned int	read;
				unsigned int	mul;
				unsigned int	setup;
				unsigned int	pulse;
				unsigned int	cycle;
				unsigned int	mode;
				unsigned int	addr;
				unsigned char	cs;


	addr = (unsigned int)paddr;
	switch ((addr >> 26) & 0x0f) {
		case 0x0:														 /* Addr = 0x00000000,	64MB, EBI SRAM CS0.					*/
			 cs = 0;
			 break;

		case 0x1:														 /* Addr = 0x04000000,	64MB, EBI SRAM CS4.					*/
			 cs = 4;
			 break;

		case 0x2:														 /* Addr = 0x08000000,	64MB, EBI SRAM CS2.					*/
			 cs = 2;
			 break;

		case 0x3:														 /* Addr = 0x0C000000,	64MB, EBI SRAM CS3.					*/
			 cs = 3;
			 break;

		case 0x4:														 /* Addr = 0x10000000, 256MB, EBI SRAM/SDRAM CS1.			*/
		case 0x5:
		case 0x6:
		case 0x7:
			 cs = 1;
			 break;

		case 0x8:														 /* Addr = 0x20000000,	64MB, EBI SRAM CS5.					*/
			 cs = 5;
			 break;

		default:
			 return;
	}

	switch (settings->BusWidth) {
		case 1:
			 mode = (0 & SMC_DBW_MASK) << SMC_DBW_OFFSET;
			 break;

		case 2:
			 mode = (1 & SMC_DBW_MASK) << SMC_DBW_OFFSET;
			 break;

		case 4:
			 mode = (2 & SMC_DBW_MASK) << SMC_DBW_OFFSET;
			 break;

		default:
			 return;
	}


	if (hsb_clock == 0) {
		return;
	}
																		/* Cycles = x / T = x * f = ...							 */
																		/* ... = ((x * 10^9) * ((f * 65536) / 10^9)) / 65536 = ...	*/
																		/* ... = ((x * 10^9) * (((f / 10^4) * 2^16) / 10^5)) / 2^16 */
	mul	 = hsb_clock;
	mul	/= 10000;
	mul <<= 16;
	mul	/= 100000;

#define ns2cyc(x) ((((x) * mul) + 65535) >> 16)

	setup = (((ns2cyc(settings->NWESetup)		& SMC_NWE_SETUP_MASK)		<< SMC_NWE_SETUP_OFFSET)	|
			 ((ns2cyc(settings->NCSWriteSetup)	& SMC_NCS_WR_SETUP_MASK)	<< SMC_NCS_WR_SETUP_OFFSET) |
			 ((ns2cyc(settings->NRDSetup)		& SMC_NRD_SETUP_MASK)		<< SMC_NRD_SETUP_OFFSET)	|
			 ((ns2cyc(settings->NCSReadSetup)	& SMC_NCS_RD_SETUP_MASK)	<< SMC_NCS_RD_SETUP_OFFSET));
	pulse = (((ns2cyc(settings->NWEPulse)		& SMC_NWE_PULSE_MASK)		<< SMC_NWE_PULSE_OFFSET)	|
			 ((ns2cyc(settings->NCSWritePulse)	& SMC_NCS_WR_PULSE_MASK)	<< SMC_NCS_WR_PULSE_OFFSET) |
			 ((ns2cyc(settings->NRDPulse)		& SMC_NRD_PULSE_MASK)		<< SMC_NRD_PULSE_OFFSET)	|
			 ((ns2cyc(settings->NCSReadPulse)	& SMC_NCS_RD_PULSE_MASK)	<< SMC_NCS_RD_PULSE_OFFSET));
	cycle = (((ns2cyc(settings->WriteCycle)		& SMC_NWE_CYCLE_MASK)		<< SMC_NWE_CYCLE_OFFSET)	|
			 ((ns2cyc(settings->ReadCycle)		& SMC_NRD_CYCLE_MASK)		<< SMC_NRD_CYCLE_OFFSET));

	if (settings->NRDControlled > 0) {
		mode |= SMC_READ_MODE_MASK << SMC_READ_MODE_OFFSET;
	}
	if (settings->NWEControlled > 0) {
		mode |= SMC_WRITE_MODE_MASK << SMC_WRITE_MODE_OFFSET;
	}
	if (settings->ByteWrite > 0) {
		mode |= SMC_BAT_MASK << SMC_BAT_OFFSET;
	}
	if (settings->TDFOptimization > 0) {
		mode |= SMC_TDF_MODE_MASK << SMC_TDF_MODE_OFFSET;
	}
	if (settings->TDFCycles < 16) {
		mode |= (settings->TDFCycles & SMC_TDF_CYCLES_MASK) << SMC_TDF_CYCLES_OFFSET;
	}

	AVR32_PM_HSBMASK_reg |= 1 <<	0;									/* Enable EBI HSB clock. */
	AVR32_PM_PBBMASK_reg |= 1 << 13;									/* Enable SMC PBB clock. */

	AVR32_SMC_MODE_reg(cs)	= mode;
	AVR32_SMC_CYCLE_reg(cs) = cycle;
	AVR32_SMC_PULSE_reg(cs) = pulse;
	AVR32_SMC_SETUP_reg(cs) = setup;

	read = AVR32_SMC_MODE_reg(cs);										/* I/O Barrier. */

	ret[0] = mode;
	ret[1] = cycle;
	ret[2] = pulse;
	ret[3] = setup;
}

